package cert;

import java.io.*;
import java.security.*;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;

import java.math.BigInteger;
import java.util.Date;
import java.util.Enumeration;

import javax.security.auth.x500.X500Principal;

import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.x509.X509V3CertificateGenerator;
import org.bouncycastle.x509.extension.AuthorityKeyIdentifierStructure;
import org.bouncycastle.x509.extension.SubjectKeyIdentifierStructure;

import org.bouncycastle.jce.PKCS10CertificationRequest;
import org.bouncycastle.openssl.PEMWriter;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;

import org.bouncycastle.asn1.*;



public class TestCA {

	public static PKCS10CertificationRequest generateRequest(KeyPair pair) 
	throws Exception {
		
		return new PKCS10CertificationRequest(
				"SHA1withRSA",
				new X500Principal("CN=Requested Test Cerificate"),
				pair.getPublic(),
				null,
				pair.getPrivate());
	}
	
	public static void exportCertificateRequest(PKCS10CertificationRequest req, String filename)
	throws Exception{
		PEMWriter pemWr = new PEMWriter(new OutputStreamWriter(new FileOutputStream(filename)));
		pemWr.writeObject(req);
		pemWr.close();
	}
	
	
	public static X509Certificate[] buildChain(PKCS10CertificationRequest request, KeyPair rootKeys) 
	throws Exception{
		
		// Create a root certificate
		X509Certificate rootCert = CreateSelfCertificate.generateV3CertificateBC(rootKeys);
		
		// Validate the certification request
		if (!request.verify("BC")){
			System.out.println("Request failed to verify");
			System.exit(1);
		}
		
		// Create the certificate using the info in the request
		X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();
		
		certGen.setSerialNumber(BigInteger.valueOf(System.currentTimeMillis()));
		certGen.setIssuerDN(rootCert.getIssuerX500Principal());
		certGen.setNotBefore(new Date(System.currentTimeMillis()));
		certGen.setNotAfter(new Date(System.currentTimeMillis() + 50000));
		certGen.setSubjectDN(request.getCertificationRequestInfo().getSubject());
		certGen.setPublicKey(request.getPublicKey("BC"));
		certGen.setSignatureAlgorithm("SHA1WithRSAEncryption");
		
		certGen.addExtension(X509Extensions.AuthorityKeyIdentifier, false, 
				new AuthorityKeyIdentifierStructure(rootCert));
		certGen.addExtension(X509Extensions.SubjectKeyIdentifier, false, 
				new SubjectKeyIdentifierStructure(request.getPublicKey("BC")));
		certGen.addExtension(X509Extensions.BasicConstraints, true, 
				new BasicConstraints(false));
		certGen.addExtension(X509Extensions.KeyUsage, true, 
				new KeyUsage(KeyUsage.digitalSignature | KeyUsage.keyEncipherment));
		certGen.addExtension(X509Extensions.ExtendedKeyUsage, true, 
				new ExtendedKeyUsage(KeyPurposeId.id_kp_serverAuth));
		
		
		
		
		// Extract the extension request attribute
	
		ASN1Set attributes = request.getCertificationRequestInfo().getAttributes();
		
		for(int i=0; i<attributes.size(); i++){
			Attribute attr = Attribute.getInstance(attributes.getObjectAt(i));
			
			// Process extension request
			if(attr.getAttrType().equals(PKCSObjectIdentifiers.pkcs_9_at_extensionRequest)){
				X509Extensions extensions = X509Extensions.getInstance(attr.getAttrValues().getObjectAt(0));
				Enumeration e = extensions.oids();
				while(e.hasMoreElements()){
					DERObjectIdentifier oid = (DERObjectIdentifier)e.nextElement();
					X509Extension ext = extensions.getExtension(oid);
					
					certGen.addExtension(oid, ext.isCritical(), ext.getValue().getOctets());
				}
			}
		}

		X509Certificate issuedCert = certGen.generate(rootKeys.getPrivate());
		return new X509Certificate[] { issuedCert, rootCert };
	}
	
	public static void exportChain(X509Certificate[] certs, String[] filename)
	throws Exception{
		for (int i=0; i<filename.length; i++){
			PEMWriter pemWr = new PEMWriter(new OutputStreamWriter(new FileOutputStream(filename[i])));
			pemWr.writeObject(certs[i]);
			pemWr.close();	
		}		
	}
	
	public static void main(String[] args){
		
		// modificare la stringa path
		String path = ""; 
		String filename = path + "myCertReq.p10";
		
		String chain1 = path + "issuedCert.crt";
		String chain2 = path + "root.crt";

		try {
			
			/*******************************************************/
			
			// Building keys
			System.out.println("Building keys...");
			KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA", "BC");
			keyPairGen.initialize(1024);
			KeyPair keys = keyPairGen.generateKeyPair();
			
			// Building the CertificateRequest
			System.out.println("Building certificate request...");
			PKCS10CertificationRequest request = generateRequest(keys);
			
			// Exporting the CertificateRequest
			System.out.println("Exporting certificate request...");
			exportCertificateRequest(request, filename);
			System.out.println("CertificateRequest exported.");
			
			/*******************************************************/

			
			// Certificate Authority
			
			// Building keys
			System.out.println("\nBuilding CA keys...");
			KeyPairGenerator keyPairGenCA = KeyPairGenerator.getInstance("RSA", "BC");
			keyPairGenCA.initialize(1024);
			KeyPair keysCA = keyPairGenCA.generateKeyPair();
			
			// Building Certificate chain
			System.out.println("(CA) Building certificate chain...");
			X509Certificate[] chain = buildChain(request, keysCA);
			System.out.println("(CA) Certificate chain built.");	
			
			// Exporting the Certificate chain
			System.out.println("(CA) Exporting certificate chain...");
			exportChain(chain, new String[] { chain1, chain2 });
			System.out.println("Certificate chain exported.");
			
		}
		
		catch(Exception e){
			e.printStackTrace();
		}	
		
	}
	
	
	
}

